BUILTIN(__builtin_arm_cde_cx1, "", "ncU")
BUILTIN(__builtin_arm_cde_cx1a, "", "ncU")
BUILTIN(__builtin_arm_cde_cx1d, "", "ncU")
BUILTIN(__builtin_arm_cde_cx1da, "", "ncU")
BUILTIN(__builtin_arm_cde_cx2, "", "ncU")
BUILTIN(__builtin_arm_cde_cx2a, "", "ncU")
BUILTIN(__builtin_arm_cde_cx2d, "", "ncU")
BUILTIN(__builtin_arm_cde_cx2da, "", "ncU")
BUILTIN(__builtin_arm_cde_cx3, "", "ncU")
BUILTIN(__builtin_arm_cde_cx3a, "", "ncU")
BUILTIN(__builtin_arm_cde_cx3d, "", "ncU")
BUILTIN(__builtin_arm_cde_cx3da, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1_u32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1a_u32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1d_u64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1da_u64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1q_m_f16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1q_m_f32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1q_m_s16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1q_m_s32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1q_m_s64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1q_m_s8, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1q_m_u16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1q_m_u32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1q_m_u64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1q_m_u8, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1q_u8, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1qa_f16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1qa_f32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1qa_m_f16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1qa_m_f32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1qa_m_s16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1qa_m_s32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1qa_m_s64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1qa_m_s8, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1qa_m_u16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1qa_m_u32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1qa_m_u64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1qa_m_u8, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1qa_s16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1qa_s32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1qa_s64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1qa_s8, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1qa_u16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1qa_u32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1qa_u64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx1qa_u8, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2_u32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2a_u32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2d_u64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2da_u64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_f16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_f32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_m_impl_f16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_m_impl_f32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_m_impl_s16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_m_impl_s32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_m_impl_s64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_m_impl_s8, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_m_impl_u16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_m_impl_u32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_m_impl_u64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_m_impl_u8, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_s16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_s32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_s64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_s8, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_u16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_u32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_u64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_u8, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_u8_f16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_u8_f32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_u8_s16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_u8_s32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_u8_s64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_u8_s8, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_u8_u16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_u8_u32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_u8_u64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2q_u8_u8, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2qa_impl_f16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2qa_impl_f32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2qa_impl_s16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2qa_impl_s32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2qa_impl_s64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2qa_impl_s8, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2qa_impl_u16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2qa_impl_u32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2qa_impl_u64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2qa_impl_u8, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2qa_m_impl_f16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2qa_m_impl_f32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2qa_m_impl_s16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2qa_m_impl_s32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2qa_m_impl_s64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2qa_m_impl_s8, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2qa_m_impl_u16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2qa_m_impl_u32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2qa_m_impl_u64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx2qa_m_impl_u8, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3_u32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3a_u32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3d_u64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3da_u64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_impl_f16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_impl_f32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_impl_s16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_impl_s32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_impl_s64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_impl_s8, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_impl_u16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_impl_u32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_impl_u64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_impl_u8, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_m_impl_f16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_m_impl_f32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_m_impl_s16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_m_impl_s32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_m_impl_s64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_m_impl_s8, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_m_impl_u16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_m_impl_u32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_m_impl_u64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_m_impl_u8, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_u8_impl_f16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_u8_impl_f32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_u8_impl_s16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_u8_impl_s32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_u8_impl_s64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_u8_impl_s8, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_u8_impl_u16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_u8_impl_u32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_u8_impl_u64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3q_u8_impl_u8, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3qa_impl_f16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3qa_impl_f32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3qa_impl_s16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3qa_impl_s32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3qa_impl_s64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3qa_impl_s8, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3qa_impl_u16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3qa_impl_u32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3qa_impl_u64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3qa_impl_u8, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3qa_m_impl_f16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3qa_m_impl_f32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3qa_m_impl_s16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3qa_m_impl_s32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3qa_m_impl_s64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3qa_m_impl_s8, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3qa_m_impl_u16, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3qa_m_impl_u32, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3qa_m_impl_u64, "", "ncU")
BUILTIN(__builtin_arm_cde_vcx3qa_m_impl_u8, "", "ncU")
BUILTIN(__builtin_arm_cde_vreinterpretq_u8_u8, "", "ncU")
